/***************************************************************************
                          masstagger.cpp  -  description
                             -------------------
    begin                : Mon Sep 22 2003
    copyright            : (C) 2003 by Avi Alkalay
    email                : avi@unix.sh
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <klocale.h>
#include <kdebug.h>
#include <kcombobox.h>
#include <ktextedit.h>
#include <qgroupbox.h>
#include <kpushbutton.h>
#include <qcheckbox.h>
#include <kfiledialog.h>

#include "musicfilenamelayout.h"
#include "musicinfo.h"
#include "masstagger.h"
#include "masstag.h" // the dialog from QT designer

MassTagger::MassTagger(KFileItemList &_fileList, FilenameMiner *_miner, QWidget *_parent) {
	parent=_parent;
	fileList=_fileList;
	miner=_miner;

	userDialog();
}

MassTagger::~MassTagger() {
	delete massTaggerDialog;
}



void MassTagger::userDialog() {
	QString fromFilename(i18n("[Guess from filename]"));
	QString empty;
	QString title;

	// Create it
	massTaggerDialog=new MassTaggerDialog(parent);

	// Connect to the method that lets the user copy ID3s from other file
	connect(massTaggerDialog->copyTags,SIGNAL(clicked()),SLOT(getTemplateFile()));

	// Connect dialog results to some actions
	connect(massTaggerDialog->ok,SIGNAL(clicked()),SLOT(accept()));
	connect(massTaggerDialog->cancel,SIGNAL(clicked()),SLOT(finish()));


	title=massTaggerDialog->groupID3->title().arg(fileList.count());
	massTaggerDialog->groupID3->setTitle(title);
	
	massTaggerDialog->iTitle->insertItem(empty,0);
	massTaggerDialog->iArtist->insertItem(empty,0);
	massTaggerDialog->iTrack->insertItem(empty,0);
	massTaggerDialog->iAlbum->insertItem(empty,0);

	// Set some default initial values in some fields
	massTaggerDialog->iTitle->insertItem(fromFilename,1);
	massTaggerDialog->iArtist->insertItem(fromFilename,1);
	massTaggerDialog->iTrack->insertItem(fromFilename,1);
	massTaggerDialog->iAlbum->insertItem(fromFilename,1);

	// Make the dialog appear
	massTaggerDialog->exec();
}




void MassTagger::accept() {
	KFileItem *file;
	MusicInfo dlg=musicInfoFromDialog(); // Music info from dialog input fields

	for (file=fileList.first();file;file=fileList.next()) {
		MusicInfo org; // Info from the original file
		MusicInfo fromFilename; // Info guessed from the filename
		KFileMetaInfo metaInfo=file->metaInfo();

		// Get info from original file
		org=MusicInfo(*file);

		// Guess some info from filename
		fromFilename=miner->mine(file->url().fileName());
		cleanFromFilename(fromFilename);

		// Merge objects
		org.merge(dlg);
		org.merge(fromFilename);

		org.updateMetaInfo(metaInfo);
		metaInfo.applyChanges();
	}
}





/** Returns the MusicInfo object based on the user input from the dialog. Unchecked items in the dialog will make empty fields in the MusicInfo object. */
MusicInfo MassTagger::musicInfoFromDialog(){
		MusicInfo dlg;
		
		// This first 4 gives the option to guess from filename
		if (massTaggerDialog->cTitle->isChecked() && massTaggerDialog->iTitle->currentItem()!=1)
			dlg.title=massTaggerDialog->iTitle->currentText();

		if (massTaggerDialog->cArtist->isChecked() && massTaggerDialog->iArtist->currentItem()!=1)
			dlg.artist=massTaggerDialog->iArtist->currentText();

		if (massTaggerDialog->cAlbum->isChecked() && massTaggerDialog->iAlbum->currentItem()!=1)
			dlg.album=massTaggerDialog->iAlbum->currentText();

		if (massTaggerDialog->cTrack->isChecked() && massTaggerDialog->iTrack->currentItem()!=1)
			dlg.track=massTaggerDialog->iTrack->currentText().toInt();

		if (massTaggerDialog->cYear->isChecked())
			dlg.year=massTaggerDialog->iYear->currentText().toInt();

		if (massTaggerDialog->cComment->isChecked())
			dlg.comment=massTaggerDialog->iComment->text();

		return dlg;
}


/** Write property of FilenameMiner * miner. */
void MassTagger::setMiner( FilenameMiner * _newVal) {
	miner = _newVal;
}

/** Remove the values that the user don't want guessed from the filename. */
void MassTagger::cleanFromFilename(MusicInfo &dlg) {
		// This first 4 gives the option to guess from filename

		if (!massTaggerDialog->cTitle->isChecked() || massTaggerDialog->iTitle->currentItem()!=1)
			dlg.title=QString::null;

		if (!massTaggerDialog->cArtist->isChecked() || massTaggerDialog->iArtist->currentItem()!=1)
			dlg.artist=QString::null;

		if (!massTaggerDialog->cAlbum->isChecked() || massTaggerDialog->iAlbum->currentItem()!=1)
			dlg.album=QString::null;

		if (!massTaggerDialog->cTrack->isChecked() || massTaggerDialog->iTrack->currentItem()!=1)
			dlg.track=-1;

		if (!massTaggerDialog->cYear->isChecked())
			dlg.year=-1;

		if (!massTaggerDialog->cComment->isChecked())
			dlg.comment=QString::null;
}


/** A wrapper to the destructor in the form of a slot. */
void MassTagger::finish() {
	delete this;
}


/** Called when user chooses to copy ID3 tags from other file. Fills the dialog with the values encontered. */
void MassTagger::getTemplateFile() {
	KURL choosed;

	choosed=KFileDialog::getOpenURL(QString::null,"audio/x-mp3 application/x-ogg",
		massTaggerDialog,i18n("Select file to copy ID3 tags from"));

	if (!choosed.isEmpty()) {
		KFileItem fi(choosed,QString::null,0);
		MusicInfo info(fi);

		if (!info.artist.isEmpty())
			massTaggerDialog->iArtist->insertItem(info.artist);
		if (!info.title.isEmpty())
			massTaggerDialog->iTitle->insertItem(info.title);
		if (!info.album.isEmpty())
			massTaggerDialog->iAlbum->insertItem(info.album);
		if (!info.comment.isEmpty())
			massTaggerDialog->iComment->setText(info.comment);
		if (info.track!=-1)
			massTaggerDialog->iTrack->insertItem(QString::number(info.track));
		if (info.year!=-1)
			massTaggerDialog->iYear->insertItem(QString::number(info.year));
	}
}
