/***************************************************************************
                          musicindexgenerator.cpp  -  description
                             -------------------
    begin                : Sun Jun 1 2003
    copyright            : (C) 2003 by Avi Alkalay
    email                : avi@unix.sh
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <math.h>
#include <kdebug.h>
#include <kglobal.h>
#include <klocale.h>
#include <kio/job.h>
#include <kio/jobclasses.h>
#include <qmap.h>
#include <qregexp.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qcheckbox.h>
#include <kstandarddirs.h>
#include <kzip.h>
#include <kcombobox.h>
#include <klineedit.h>
#include <kpushbutton.h>
#include <ksqueezedtextlabel.h>

#include "musicindexgenerator.h"
#include "dialogindexerinfo.h"




void MusicIndexGenerator::subConstructor(const KURL &_base, QString &_htmlTemplate, QString &_coverTemplate) {
	KIO::ListJob *job;

	base=_base;
	root=QString(i18n("--Root Folder--"));
	CDName=clean(base.fileName());
	htmlTemplate=_htmlTemplate;
	coverTemplate=_coverTemplate;
	userAccepted=dialogFinished=haveAllFilesInfo=false;

	// Start listing and analyzing music files in background
	job=KIO::listRecursive(base,false);
	connect(job, SIGNAL(entries(KIO::Job *,const KIO::UDSEntryList&)),
		SLOT(buildMusicListFase1(KIO::Job *,const KIO::UDSEntryList&)));
	connect(job,SIGNAL(result(KIO::Job *)),
		SLOT(buildMusicListFase2(KIO::Job *)));

	// kdDebug(0) << "Index engine started for " << base.prettyURL() << endl;

	// In the mean time, ask user some thing regarding cover and HTML templates
	userDialog();
}


/** Called when the user accepts the DialogIndexerInfo. Will update some internal vars to correctly generate the indexes. */
void MusicIndexGenerator::updateFromDialog() {
	CDName=dialogIndexerInfo->CDName->text();
	htmlTemplate=dialogIndexerInfo->htmlTemplate->currentText();
	coverTemplate=dialogIndexerInfo->coverTemplate->currentText();
	if (dialogIndexerInfo->m3uToggle->isChecked())
		m3uFilename=dialogIndexerInfo->m3uFilename->text();
	userAccepted=true;
	setDialogFinished();
}


/** Details of creating the user supplied info dialog. */
void MusicIndexGenerator::userDialog() {
	QStringList htmlDirs;
	QStringList coverDirs;
	
	dialogIndexerInfo=new DialogIndexerInfo(parent);

	// Connect dialog results to some actions
	connect(dialogIndexerInfo->okButton,SIGNAL(clicked()),SLOT(updateFromDialog()));
	connect(dialogIndexerInfo->cancelButton,SIGNAL(clicked()),SLOT(setDialogFinished()));

	// Search for cover and HTML templates
	htmlDirs=getSubItems("musicman/htmlTemplates");
	coverDirs=getSubItems("musicman/coverTemplates");

	// Tell the dialog about the templates, setting the comboboxes
	dialogIndexerInfo->htmlTemplate->insertStringList(htmlDirs);
	dialogIndexerInfo->coverTemplate->insertStringList(coverDirs);
	dialogIndexerInfo->CDName->setText(CDName);

	// Make the dialog appear
	kdDebug(0) << "Indexer dialog appearing..." << endl;
	dialogIndexerInfo->exec();
}





MusicIndexGenerator::MusicIndexGenerator(const KURL &_base, QWidget *_parent, QString &_htmlTemplate, QString &_coverTemplate) {
	parent=_parent;
	subConstructor(_base,_htmlTemplate,_coverTemplate);
}









MusicIndexGenerator::MusicIndexGenerator(const KURL &_base, QWidget *_parent) {
	QString def("default");
	QString defCover("default");
	parent=_parent;
	subConstructor(_base,def,defCover);
}








MusicIndexGenerator::~MusicIndexGenerator() {
	kdDebug(0) << "Destructor for " << base.prettyURL() << endl;
}









/** Called after the list job finishes. It will build the musicList and folderList vars. */
void MusicIndexGenerator::buildMusicListFase1(KIO::Job *job, const KIO::UDSEntryList &list) {
	KIO::UDSEntryListConstIterator it = list.begin();
	KIO::UDSEntryListConstIterator end = list.end();


	// To stop the anoying compiler warning "unused parameter" message
	job->window();
	
	kdDebug(0) << "Listing fase 1 for " << base.prettyURL() << endl;

	// First put everything in a list...
	while (it != end) {
		KIO::UDSEntry::ConstIterator it2 = (*it).begin();
		for( ; it2 != (*it).end(); it2++ ) {
			if ((*it2).m_uds == KIO::UDS_NAME)
				fileNames+=(*it2).m_str;
				kdDebug(0) << "buildMusicListFase1: Included file " << (*it2).m_str << endl;
			break;
		}
		++it;
	}

	kdDebug(0) << "Listing fase 1 finished for " << base.prettyURL() << endl;

  // buildMusicListFase2 will be called to build our complex structure.
	// See constructor for the signal that calls it.
}









void MusicIndexGenerator::buildMusicListFase2(KIO::Job *job) {
	QStringList keys;
	KFileItemList *currentFolder=0L;
	MusicFolderInfo *currentFolderInfo=0L;
	QStringList::Iterator fileName;

	// To stop the anoying compiler warning "unused parameter" message
	job->window();

	
	kdDebug(0) << "Listing fase 2 for " << base.prettyURL() << endl;

	// Sort the file paths we took from fase 1
	fileNames.sort();

	// ...then put them in our high-level structures
	for (fileName=fileNames.begin(); fileName!=fileNames.end(); ++fileName) {
		if (isMusic(*fileName)) {
			KFileItem *currentItem;
			QString folder=dirName(*fileName); // get relative folder
			KURL url;

			kdDebug(0) << "Detected "<< *fileName << " in " << folder << endl;
			if (! musicList.contains(folder)) {
				// KFileItemList for this folder was not created yet. Create it.
				// If the folder doesn't exist yet, just referencing it will create it.
				musicList[folder].setAutoDelete(true);
			}

			currentFolder=&musicList[folder];
			currentFolderInfo=&foldersInfo[folder];

			// Create our file item
			url.setPath(base.path(1)+*fileName); // safer for filenames containing things like '?'
			currentItem=new KFileItem(KFileItem::Unknown, KFileItem::Unknown, url);

			// Merge file's statistics to its folder statistics
			currentFolderInfo->add(*currentItem);

			// Add it to the KFileItemList (its folder)
			currentFolder->append(currentItem);
		}
	}

	// Finalize the root dir statistics adding subfolders statistics
	keys=foldersInfo.keys();
	for (QStringList::Iterator it2=keys.begin(); it2!=keys.end(); ++it2) {
		if ((*it2) != root)
			foldersInfo[root].add(foldersInfo[(*it2)]);
/*
		KFileItem *fit;
		kdDebug(126) << "\nFolder " << (*it2) << endl;
		currentFolder=&musicList[(*it2)];
		fit=currentFolder->first();
		while (fit) {
			kdDebug(126) << fit->permissionsString() << ": " << fit->url().prettyURL() << endl;
			fit=currentFolder->next();
*/
	}
	//kdDebug(0) << "Emiting fase 2 finished signal" << endl;
	infoBuilderFinished();

	kdDebug(0) << "Listing fase 2 finished for " << base.prettyURL() << endl;

}




void MusicIndexGenerator::infoBuilderFinished() {
	haveAllFilesInfo=true;
	createAll();
}









/** Will create all necessary static files for a nice CD. Like images, the basic diretories resource and info, CD icon, Windows autorun.inf, etc */
void MusicIndexGenerator::prepareCDLayout() {
	KURL::List layout;
	QStringList resDirs;
	KIO::Job *job;

	// Find CD layout template dir
	resDirs=KGlobal::dirs()->findDirs("data","musicman/cdRootLayout");
	if (!resDirs.count()) return; // No CD template dir found.

	QDir templateDir(resDirs[0],QString::null);
	QStringList templateFiles = templateDir.entryList();
	for (QStringList::ConstIterator i = templateFiles.begin() ;
			i != templateFiles.end();
			++i) {
		if ((*i) == ".." || (*i) == ".") continue;
		KURL item(templateDir.path()+"/"+(*i));
		kdDebug() << "CD root layout file: " << item.prettyURL() << endl;
		// Might need to prepend the dirname, too, in
		// which case use templateDir::path()+(*i)
		layout.append(item);
	}
	
	// Copy CD template to konqueror-selected folders.
	job=KIO::copy(layout,base,false);
	// After copy is finished, start creating the indexes.
	connect(job,SIGNAL(result(KIO::Job *)),SLOT(createIndexes()));
}














/** Create the nice HTML index

This method has 5 stages:

1. Find the HTML template requested in htmlTemplate variable
2. Copy all static files from the template to the CD root folder
3. Browse the music information structures we've built in other methods
4. Substitute the macros in the HTML template files with strings we built
5. Save the executed macros in the CD root folder

This method searches for the following macros, that will be substituted by pure
JavaScript code:

_TITLE_
Substituted by the CD title. Usually the folder name.

_TOC_


_MEDIA_


_STATISTICS_

 */
void MusicIndexGenerator::createHTML() {
	KURL::List toCopy,toProcess;
	QStringList resDirs;
	QRegExp toProcessFilter("\\.html$|\\.htm$|\\.js$",false);
	
	QString media;
	QString toc;

	if (htmlTemplate.at(0) == '/') {
		// It is an absolute path
		resDirs.append(htmlTemplate);
	} else {
		// It is just a name, so find it in our default directories
		resDirs=KGlobal::dirs()->findDirs("data",
			"musicman/htmlTemplates/" + htmlTemplate);
	}

	if (!resDirs.count()) {
		// We don't have a template to work with, so just return.
		return;
	}

	kdDebug(126) << "HTML template: " << resDirs[0] << endl;


//	n = scandir((*(QStringList::Iterator)resDirs.begin()).ascii(),
//			&namelist, 0, alphasort);


	// List all files in first template directory
	QDir templateDir(resDirs[0]);
	QStringList templateFiles = templateDir.entryList();
	QStringList::ConstIterator i = templateFiles.begin();
	while (i!=templateFiles.end()) {
		if ((*i) != ".." && (*i) != ".") {
			KURL entry((*(QStringList::Iterator)resDirs.begin()) + "/" + (*i));
		
			if (toProcessFilter.search(*i)!=-1) {
				// We need to process this file.
				toProcess.append(entry);
				kdDebug(126) << "HTML to process: " << entry.prettyURL() << endl;
			} else {
				toCopy.append(entry);
				kdDebug(126) << "HTML to copy: " << entry.prettyURL() << endl;
			}
		}
		++i;
	}
	
/*	
	// Build the KURL::List of template items to copy
	while (n--) {
		KURL entry;
		QString found(namelist[n]->d_name);

		if (found!=".." && found!=".") {
			entry=KURL((*(QStringList::Iterator)resDirs.begin()) + "/" + found);
			if (toProcessFiles.search(found)!=-1) {
				// We need to process this file.
				toProcess.append(entry);
				// kdDebug(126) << "to process: " << entry.prettyURL() << "\n";
			} else {
				toCopy.append(entry);
			}
			free(namelist[n]);
		}
	}
	free(namelist);*/

	// Copy static HTML files from template to user-selected folder
	KIO::copy(toCopy,base,false);
	
	// Process the possible template files
	KFileItemList *currentFolder=0L;
	QStringList keys;

	int n=0;    // used as a folder HTML reference index (#dir0, #dir1, #dirn)
	keys=foldersInfo.keys();
	unsigned int indexInFolder=0,indexInCD=0;
	for (QStringList::Iterator it=keys.begin(); it!=keys.end(); ++it) {
		KFileItem *fit;
		QString jsFolderItem;

		indexInFolder=0;

		if (musicList.contains(*it))
			currentFolder=&musicList[*it];
		else continue;

		/* JavaScript content will be generated this way:

			folder(real_name, cleaned_name, items,
				playingTime, nice_playingTime, size, nice_size,
				html_reference);

			item(relative_path, cleaned_name,
				playingTime, nice_playingTime, size, nice_size,
				cd_index, folder_index, bitrate, nice_bitrate,
				artist, title, album, track, year, genre, comment);
				
			This guarantees one can manipulate it he wants just
			reimplementing the folder() and item() JavaScript functions
			inside the HTML templates.

		*/
		
		jsFolderItem+="folder(\"" + (*it) + "\",\"";
		jsFolderItem+=clean((*it)) + "\",";
		jsFolderItem+=QString::number(foldersInfo[(*it)].items) + ",";
		jsFolderItem+=QString::number(foldersInfo[(*it)].playingTime) + ",\"";
		jsFolderItem+=foldersInfo[(*it)].nicePlayingTime() + "\",";
		jsFolderItem+=KIO::number(foldersInfo[(*it)].size) + ",\"";
		jsFolderItem+=KIO::convertSize(foldersInfo[(*it)].size) + "\",\"";
		jsFolderItem+="dir" + QString::number(n) + "\");\n";

		toc+=jsFolderItem;
		media+="\n" + jsFolderItem + "preFolderContent();\n";

		fit=currentFolder->first();
		while (fit) {
			KFileMetaInfo mif=fit->metaInfo();
			QString jsFileItem;

			indexInFolder++;
			indexInCD++;

			kdDebug(0) << "createHTML: Processing file " << fit->name() <<endl;

			/*
			item(relative_path, cleaned_name,
				playingTime, nice_playingTime, size, nice_size,
				cd_index, folder_index, bitrate, nice_bitrate,
				artist, title, album, track, year, genre, comment);
			*/

			jsFileItem+="item(\"";
			if ((*it)!=root) jsFileItem+=(*it) + "/";
			jsFileItem+=fit->name() + "\",\"";
			jsFileItem+=clean(fit->name()) + "\",";
			jsFileItem+=mif.value("Length").toString() + ",\"";
			jsFileItem+=nicePlayingTime(mif.value("Length").toUInt()) + "\",";
			jsFileItem+=KIO::number(fit->size()) + ",\"";
			jsFileItem+=KIO::convertSize(fit->size()) + "\",";
			jsFileItem+=QString::number(indexInCD) + "," + QString::number(indexInFolder) + ",";
			jsFileItem+=mif.value("Bitrate").toString() + ",\"";
			jsFileItem+=mif.value("Bitrate").toString() + " Kb/s\");\n"; /*,\"";
			jsFileItem+=mif.value("Artist").toString() + "\",\"";
			jsFileItem+=mif.value("Title").toString() + "\",\"";
			jsFileItem+=mif.value("Album").toString() + "\",\"";
			jsFileItem+=mif.value("Tracknumber").toString() + "\",";
			jsFileItem+=mif.value("Year").toString() + ",\"";
			jsFileItem+=mif.value("Genre").toString() + "\",\"";
			jsFileItem+=mif.value("Comment").toString() + "\");\n"; */
			
			media+=jsFileItem;

			fit=currentFolder->next();
		}
		media+="postFolderContent();\n";
		++n;  // used as a folder HTML reference index (#dir0, #dir1, #dirn)
	}


	/*
		// Generate JavaScript in this format for general CD statistics:
		statistics(items, playingTime, nice_playingTime, subFolders, size, niceSize);

		// Generate date and time in this format:
		generated(day,month,year,hour,min,sec);

	*/

	QString jsStatistics;
	jsStatistics+="statistics(\"" + QString::number(foldersInfo[root].items) + "\",\"";
	jsStatistics+=QString::number(foldersInfo[root].playingTime) + "\",\"";
	jsStatistics+=foldersInfo[root].nicePlayingTime() + "\",\"";
	jsStatistics+=QString::number(foldersInfo[root].subFolders) + "\",\"";
	jsStatistics+=KIO::number(foldersInfo[root].size) + "\",\"";
	jsStatistics+=KIO::convertSize(foldersInfo[root].size) + "\");\n";

	
	// Now read template by template and substitute all macros we just built
	for (KURL::List::iterator uit=toProcess.begin(); uit!=toProcess.end(); ++uit) {
		QFile file((*uit).path());
		QString templateContent;
		QTextStream stream;


		// kdDebug(0) << "createHTML: Processing template " << (*uit).path() <<endl;

		// Open and read the template
		file.open(IO_ReadOnly);
		stream.setDevice(&file);
		templateContent=stream.read();
		stream.unsetDevice();
		file.close();

		
		// Replace macros
		templateContent.replace("_TITLE_",CDName);
		templateContent.replace("_MEDIA_",media);
		templateContent.replace("_TOC_",toc);
		templateContent.replace("_STATISTICS_",jsStatistics);
		

		// kdDebug(0) << "createHTML: Writing to file " << base.path(1) + (*uit).fileName() << endl;

				
		// Write final file in target destination
		file.setName(base.path(1) + (*uit).fileName());
		file.open(IO_WriteOnly);
		stream.setDevice(&file);
		stream << templateContent;
		stream.unsetDevice();
		file.close();
	}
}










/** Create the KOffice or OpenOffice document with full music files listing to be printed and used as CD cover.
Additionally, runs KOffice to let the user make final adjustements and generate PDF. */
void MusicIndexGenerator::createCover() {
	QStringList templates;
	KIO::Job *listJob;


	if (coverTemplate.at(0) == '/') {
		// It is an absolute path
		templates.append(coverTemplate);
	} else {
		// It is just a name, so find it in our default directories
		templates=KGlobal::dirs()->findDirs("data",
			"musicman/coverTemplates/" + coverTemplate);
	}

	if (!templates.count()) {
		// We don't have a template to work with, so just return.
		return;
	}

	coverTemplateDir=*(QStringList::Iterator)templates.begin();

	// Get the cover filename extension.
	// The cover template dirname must have an extension like .kwd (KOffice-like) or .sxw (Open/StarOffice-like)
	QRegExp ext("\\.(.{1,4})$");
	ext.search(coverTemplateDir);
	coverFilenameExtension=ext.cap(1);

	// List cover template files
	listJob=KIO::listRecursive(coverTemplateDir,false);

	connect(listJob,SIGNAL(entries(KIO::Job *,const KIO::UDSEntryList&)),
		SLOT(collectCoverFiles(KIO::Job *,const KIO::UDSEntryList&)));
		
	connect(listJob,SIGNAL(result(KIO::Job *)),
		SLOT(createCoverStage2(KIO::Job *)));
}



void MusicIndexGenerator::collectCoverFiles(KIO::Job *job,const KIO::UDSEntryList &list) {
	KIO::UDSEntryListConstIterator it = list.begin();
	KIO::UDSEntryListConstIterator end = list.end();


	// To stop the anoying compiler warning "unused parameter" message
	job->window();
	
	//kdDebug(0) << "Listing fase 1 for " << base.prettyURL() << endl;

	
	// First put everything in a list...
	while (it != end) {
		KIO::UDSEntry::ConstIterator it2 = (*it).begin();
		for( ; it2 != (*it).end(); it2++ ) {
			QString n=(*it2).m_str;
			if ((*it2).m_uds == KIO::UDS_NAME && n != "." && n != "..") {
				KFileItem fit((*it),KURL(coverTemplateDir + "/" + n));
				fit.refresh();
				
				// We are interested only in real files, not directories.
				// Stop here if it is a directory.
				if (fit.isDir()) continue;
				
				if (n.find("atom")!=-1) {
					// Inside the .atoms folder we have the items templates. Process them.
					QFile tpl(fit.url().path());
					QByteArray reader;

					// Read item template from file
					tpl.open(IO_ReadOnly);
					reader=tpl.readAll();
					tpl.close();

					// Select on which variable this template should go to
					if (n.find("coverFolder")!=-1) {
						coverFolderTemplate=QString::fromUtf8(reader,reader.size());
					} else if (n.find("listFolder")!=-1) {
						coverListFolderTemplate=QString::fromUtf8(reader,reader.size());
					} else if (n.find("listSong")!=-1) {
						coverListSongTemplate=QString::fromUtf8(reader,reader.size());
					}
					tpl.close();
				} else {
					coverFiles+=n;
				}
				break;
			}
		}
		++it;
	}
}


/** Second stage of the cover creation. Will be called after the template copy job finishes. */
void MusicIndexGenerator::createCoverStage2(KIO::Job *job) {
	KFileItemList *currentFolder=0L;
	unsigned int folderIndex=0,indexInFolder=0,indexInCD=0;

	// To stop the anoying compiler warning "unused parameter" message
	job->window();
	
	// kdDebug(0) << "Cover stage 2 " << endl;

	QString toc,media,temp;
	QStringList keys=foldersInfo.keys();
	for (QStringList::Iterator it=keys.begin(); it!=keys.end(); ++it) {
		// Walk folder by folder
		
		indexInFolder=0;
		++folderIndex;
		
		if (musicList.contains(*it))
			currentFolder=&musicList[*it];
		else continue;

		temp=coverFolderTemplate;
		temp.replace("_FOLDER_",clean(*it));
		temp.replace("_ITEMS_",QString::number(foldersInfo[(*it)].items));
		temp.replace("_TIME_",QString::number(foldersInfo[(*it)].playingTime));
		temp.replace("_NICETIME_",foldersInfo[(*it)].nicePlayingTime());
		temp.replace("_SIZE_",KIO::number(foldersInfo[(*it)].size));
		temp.replace("_NICESIZE_",KIO::convertSize(foldersInfo[(*it)].size));
		temp.replace("_FOLDERINDEX_",QString::number(folderIndex));
		toc+=temp;
		
		temp=coverListFolderTemplate;
		temp.replace("_FOLDER_",clean(*it));
		temp.replace("_ITEMS_",QString::number(foldersInfo[(*it)].items));
		temp.replace("_TIME_",QString::number(foldersInfo[(*it)].playingTime));
		temp.replace("_NICETIME_",foldersInfo[(*it)].nicePlayingTime());
		temp.replace("_SIZE_",KIO::number(foldersInfo[(*it)].size));
		temp.replace("_NICESIZE_",KIO::convertSize(foldersInfo[(*it)].size));
		temp.replace("_FOLDERINDEX_",QString::number(folderIndex));
		media+=temp;
		
		KFileItem *fit=currentFolder->first();
		while(fit) {
			// Process each file inside a folder
			KFileMetaInfo mif=fit->metaInfo();
			
			indexInFolder++;
			indexInCD++;
			
			temp=coverListSongTemplate;
			temp.replace("_TITLE_",clean(fit->name()));
			temp.replace("_TIME_",mif.value("Length").toString());
			temp.replace("_NICETIME_",nicePlayingTime(mif.value("Length").toUInt()));
			temp.replace("_SIZE_",KIO::number(fit->size()));
			temp.replace("_NICESIZE_",KIO::convertSize(fit->size()));
			temp.replace("_CDINDEX_",QString::number(indexInCD));
			temp.replace("_FOLDERINDEX_",QString::number(indexInFolder));
			temp.replace("_BITRATE_",mif.value("Bitrate").toString());
			temp.replace("_NICEBITRATE_",mif.value("Bitrate").toString() + " Kb/s");
			media+=temp;
			
			fit=currentFolder->next();
		}
	}
	
	// Now we'll create the KOffice or OpenOffice cover file. It is a Zip file.
	QCString raw;
	QStringList::Iterator coverFileName;
	QString filename("info/cover");
	if (!coverFilenameExtension.isEmpty()) filename+="." + coverFilenameExtension;
	QFile::remove(base.path(1)+filename); // remove any axisting file
	KZip zipCover(base.path(1)+filename);
	zipCover.open(IO_ReadWrite);

	for (coverFileName=coverFiles.begin(); coverFileName!=coverFiles.end(); coverFileName++) {
		QByteArray fileContent;
		QFile coverFile(coverTemplateDir + "/" + (*coverFileName));
		
		coverFile.open(IO_ReadOnly);
		fileContent=coverFile.readAll();
		// kdDebug() << *coverFileName << " read: " << fileContent.size() << endl;
		coverFile.close();
		
		if ((*coverFileName).find(".xml",0,false)!=-1) {
			// A template. Process macros first.
			QString xmlTemplate=QString::fromUtf8(fileContent,fileContent.size());
			// kdDebug(0) << *coverFileName << " QString length: " << xmlTemplate.length() << endl;
			
			// Replace the CD statistics info
			xmlTemplate.replace("_STAT_ITEMS_",QString::number(foldersInfo[root].items));
			xmlTemplate.replace("_STAT_PLAYTIME_",foldersInfo[root].nicePlayingTime());
			xmlTemplate.replace("_STAT_FOLDERS_",QString::number(foldersInfo[root].subFolders));
			xmlTemplate.replace("_STAT_SIZE_",KIO::convertSize(foldersInfo[root].size));
			
			// Replace the CD title
			xmlTemplate.replace("_TITLE_",CDName);
			
			// Replace the main cover, that is the folders list
			xmlTemplate.replace("_TOC_",toc);
			
			// Replace the full music listing, generally with small fonts
			xmlTemplate.replace("_MEDIA_",media);
			
			// UTF-8 is the standard for KOffice or OpenOffice docs
			QCString content=xmlTemplate.utf8();
			fileContent.duplicate((const char *)content,content.length());
		}
		zipCover.writeFile((*coverFileName),"","",fileContent.size(),fileContent.data());
		// kdDebug() << *coverFileName << " wrote: " << fileContent.size() << endl;
	}
	zipCover.close();
}
















/** Read property of QString CDName. */
const QString& MusicIndexGenerator::getCDName(){
	return CDName;
}














/** Write property of QString CDName. */
void MusicIndexGenerator::setCDName( const QString& _newVal){
	CDName = _newVal;
}













/** Read property of KURL base. */
const KURL& MusicIndexGenerator::getbase(){
	return base;
}












/** Write property of KURL base. */
void MusicIndexGenerator::setbase( const KURL& _newVal){
	base = _newVal;
}






QString MusicIndexGenerator::baseName(const QString &path) {
	QRegExp exp("(.*)\\/(.*)");
	exp.search(path);
	return exp.cap(2);
}






QString MusicIndexGenerator::dirName(const QString &path) {
	QRegExp exp("(.*)\\/(.*)");
	if (exp.search(path) != -1) {
		return exp.cap(1);
	} else return root;
}











bool MusicIndexGenerator::isMusic(const QString &path) {
	QRegExp exp("(.*)(\\.mp3$|\\.ogg$|\\.wav$)");
	//exp.setMinimal(true);
	exp.setCaseSensitive(false);

	//kdDebug(126) << "isMusic Received: \""<< path << "\"" << endl;

	if (exp.search(path) != -1) return true;
	return false;
}








QString MusicIndexGenerator::dirName(const KIO::UDSEntry &path) {
	KIO::UDSEntry::ConstIterator it = path.begin();
	QString s;

	for( ; it != path.end(); it++ )
		if ((*it).m_uds == KIO::UDS_NAME) {
			return s=dirName((*it).m_str);
			break;
		}
	return s;
}









bool MusicIndexGenerator::isMusic(const KIO::UDSEntry &path) {
	KIO::UDSEntry::ConstIterator it = path.begin();
	for( ; it != path.end(); it++ )
		if ((*it).m_uds == KIO::UDS_NAME) {
			return isMusic((*it).m_str);
			break;
		}
	return false;
}








/** Encapsulates all index files creation methods.
Called only after the jobs created in prepareCDLayout() are over. */
void MusicIndexGenerator::createIndexes() {

	kdDebug(0) << "Creating Indexes\n";

	
	// Create HTML index
	createHTML();

	// Create plain text indexes in info/ folder
	createText();

	// Create plain text indexes in info/ folder
	createM3U();

	// Create XMedia index in info/ folder
	//createXmd();

	// Create the Cover
	createCover();

	//delete this;
}






/** Create the text indexes in info/ folder

collections.txt: Only the folder names
musicdb.txt: Full music files listing with file size, duration and quality
music.txt: Almost human readable listing of files per folder

*/
void MusicIndexGenerator::createText() {
	QFile collections,musicdb,music;
	QTextStream streamCollections,streamMusicDB,streamMusic;
	KFileItemList *currentFolder=0L;
	QStringList keys;

	// kdDebug(0) << base.path(1)+"info/collections.txt" << endl;

	collections.setName(base.path(1)+"info/collections.txt");
	musicdb.setName(base.path(1)+"info/musicdb.txt");
	music.setName(base.path(1)+"info/music.txt");
    
	collections.open(IO_WriteOnly);
	streamCollections.setDevice(&collections);
	
	musicdb.open(IO_WriteOnly);
	streamMusicDB.setDevice(&musicdb);
	
	music.open(IO_WriteOnly);
	streamMusic.setDevice(&music);

	// Walks folder by folder....
	keys=foldersInfo.keys();
	for (QStringList::Iterator it=keys.begin(); it!=keys.end(); ++it) {
		KFileItem *fit;
//		kdDebug(0) << "\nFolder " << (*it) << endl;

		if (musicList.contains(*it)) currentFolder=&musicList[*it];
		else continue;
		
		fit=currentFolder->first();
		streamCollections << clean(*it) << endl;
		streamMusic << QString("FFF ") << clean(*it) << endl;
		while (fit) {
			KFileMetaInfo mif=fit->metaInfo();

			streamMusicDB << KIO::number(fit->size()) << " " <<
				mif.value("Bitrate").toString() << " " <<
				nicePlayingTime(mif.value("Length").toUInt()) << " " <<
				(*it) + "/" + fit->name() << endl;
			
			streamMusic << clean(fit->name()) << endl;
			
			
			fit=currentFolder->next();
		}
	}

	music.close();
	musicdb.close();
	collections.close();
}







/** Cleans a raw filename to a nicer representations, substituting _s to spaces, "/" to " :: ", removing file extensions, etc */
QString MusicIndexGenerator::clean(QString s) {
	QString text=s;

	text.replace('_',' ');
	text.replace(QRegExp("\\.mp3$|\\.ogg$|\\.wav$",false),"");
	text.replace(QRegExp("\\/|\\\\|\\ \\-\\ ")," :: ");

	return text;
}









/** Returns the string for the playing time represented as hh:mm:ss */
QString MusicIndexGenerator::nicePlayingTime(unsigned long secs) {
	unsigned long h=0,m=0;
	QString formated;

	if (secs >= 3600) {
		h=(unsigned long)floor(secs/3600);
		secs-=h*3600;
	}

	if (secs >= 60) {
		m=(unsigned long)floor(secs/60);
		secs-=m*60;
	}

	if (secs<10) formated=QString("0%1").arg(secs);
	else formated=QString("%1").arg(secs);

	if (m<10) formated.prepend(QString("0%1:").arg(m));
	else formated.prepend(QString("%1:").arg(m));

	if (h) {
		if (h<10) formated.prepend(QString("0%1:").arg(h));
		else formated.prepend(QString("%1:").arg(h));
	}

	return formated;
}

/** Return the list of all resources inside the rel directories.
     Same functionality as KStandardDirs::findAllResources,
     but didn't worked for a konqueror plugin. */
QStringList MusicIndexGenerator::getSubItems(const QString &rel) {
	QStringList resDirs;
	QStringList resources;
	
	// Find templates
	resDirs=KGlobal::dirs()->findDirs("data",rel);
	if (!resDirs.count()) return resources; // No template dir found.

	for (QStringList::Iterator it=resDirs.begin(); it!=resDirs.end(); it++) {
		QDir templateDir(resDirs[0],QString::null);
		QStringList templateFiles = templateDir.entryList(QDir::Dirs);

		for (QStringList::ConstIterator found = templateFiles.begin() ;
				found != templateFiles.end();
			++found) {
			if (*found!=".." && *found!=".") {
				resources.append((*it) + *found);
			}
		}
	}
/*
			
		struct dirent **namelist;
		int n;
		n = scandir((*it).ascii(),&namelist, 0, alphasort);

		while (n--) {
			QString found(namelist[n]->d_name);

			if (found!=".." && found!=".") {
				resources.append((*it) + found);
			}
			free(namelist[n]);
		}
		free(namelist);
	}*/
	return resources;
}



/** Called when the user finished interacting the the dialog. */
void MusicIndexGenerator::setDialogFinished() {
	dialogFinished=true;
	createAll();
}


/** The starting point to create all music indexes, prepare CD layout, and destructors. */
void MusicIndexGenerator::createAll() {
	/*
		This method is called in 2 situations:

		1. The music files listing and analisys is over
		2. The user finished interacting with the dialog (see userDialog())

		It is the trigger for all other indexes (cover, HTML) generation.
		But we can only do something if both situations together are finished.
	*/

	if (dialogFinished && haveAllFilesInfo) {
		if (userAccepted==false) {
			// User rejected the dialog, so he doesn't want nothing.
			// Do nothing and destroy ourselves.
			delete this;
			return;
		}
	} else {
		// The flags indicate the context is not ready to call this method.
		// Return and wait to be called again when user accepted the dialog AND
		// we have all music files info.
		return;
	}

	// Here: OK, so all the context is ready for working.
	// Call prepareCDLayout. The index generation engines will be called from there.
	prepareCDLayout();
}


/** Create the M3U file in each folder, if solicited by user. */
void MusicIndexGenerator::createM3U() {
	KFileItemList *currentFolderList=0L;
	QStringList keys;

	if (m3uFilename.isEmpty()) return;

	// Walks folder by folder....
	keys=foldersInfo.keys();
	for (QStringList::Iterator folderIt=keys.begin(); folderIt!=keys.end(); ++folderIt) {
		if (musicList.contains(*folderIt)) currentFolderList=&musicList[*folderIt];
		else continue; // No music file(s) in this folder :-( Let's try next.

		QFile m3uFile;
		QTextStream streamM3U;
		KFileItem *fit;

		if (*folderIt == root) m3uFile.setName(base.path(1)+"/"+m3uFilename);
		else m3uFile.setName(base.path(1)+*folderIt+"/"+m3uFilename);
		m3uFile.open(IO_WriteOnly);
		streamM3U.setDevice(&m3uFile);

		streamM3U << "#EXTM3U" << endl; // Write M3U header

		fit=currentFolderList->first();
		while (fit) {
			streamM3U << fit->name() << endl;
			fit=currentFolderList->next();
		}

		m3uFile.close();
	}
}
