/***************************************************************************
                          musicinfo.cpp  -  description
                             -------------------
    begin                : Sat Jun 21 2003
    copyright            : (C) 2003 by Avi Alkalay
    email                : avi@unix.sh
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "musicinfo.h"
#include "musicfilenamelayout.h"
#include <kfileitem.h>
#include <kfilemetainfo.h>

/** Constructor from a KFileMetaInfo object */
MusicInfo::MusicInfo(KFileMetaInfo &metaInfo) {
	fromMetaInfo(metaInfo);
}

/** Constructor from a KFileItem object */
MusicInfo::MusicInfo(KFileItem &file) {
	KFileMetaInfo minfo=file.metaInfo();
	extension=MusicFilenameLayout::getExtension(file.url().fileName());
	fromMetaInfo(minfo);
}


void MusicInfo::fromMetaInfo(KFileMetaInfo &metaInfo) {
	track=metaInfo.item("Tracknumber").value().toInt();
	year=metaInfo.item("Year").value().toInt();
	artist=metaInfo.item("Artist").value().toString();
	title=metaInfo.item("Title").value().toString();
	album=metaInfo.item("Album").value().toString();
	comment=metaInfo.item("Comment").value().toString();
	bitrate=metaInfo.item("Bitrate").value().toInt();
}

MusicInfo::MusicInfo() {
	year=-1;
	track=-1;
}

MusicInfo::~MusicInfo() {
}

/** Returns true if empty. */
bool MusicInfo::isNull() {
	if (album.isNull() && artist.isNull() && title.isNull() &&
		lyrics.isNull() && comment.isNull() &&
		track<0 && year<0) return true;
	
	return false;
}

/** Updates the passed metaInfo with the MusicInfo internal info.
@param metaInfo The metaInfo that will be updated. */
void MusicInfo::updateMetaInfo(KFileMetaInfo &metaInfo) {
		if (isNull()) return;

		if (track>-1)
			metaInfo.item("Tracknumber").setValue(track);

		if (year>-1)
			metaInfo.item("Year").setValue(year);

		if (!artist.isNull())
			metaInfo.item("Artist").setValue(artist);

		if (!title.isNull())
			metaInfo.item("Title").setValue(title);

		if (!album.isNull())
			metaInfo.item("Album").setValue(album);

		if (!comment.isNull())
			metaInfo.item("Comment").setValue(comment);
}





/** Updates info with the non-null info from newInfo. The empty (empty!=null) strings are updated. */
void MusicInfo::merge(MusicInfo &newInfo) {
		if (newInfo.isNull()) return;

		if (newInfo.track>-1)
			track=newInfo.track;

		if (newInfo.year>-1)
			year=newInfo.year;

		if (!newInfo.artist.isNull())
			artist=newInfo.artist;

		if (!newInfo.title.isNull())
			title=newInfo.title;

		if (!newInfo.album.isNull())
			album=newInfo.album;

		if (!newInfo.comment.isNull())
			comment=newInfo.comment;

		if (!newInfo.extension.isNull())
			extension=newInfo.extension;

		if (!newInfo.lyrics.isNull())
			lyrics=newInfo.lyrics;

		if (newInfo.bitrate>0)
			bitrate=newInfo.bitrate;
}
